/***********************************************************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only intended for use with Renesas products.
* No other uses are authorized. This software is owned by Renesas Electronics Corporation and is protected under all
* applicable laws, including copyright laws. 
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED
* OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
* NON-INFRINGEMENT.  ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY
* LAW, NEITHER RENESAS ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE FOR ANY DIRECT,
* INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR
* ITS AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software and to discontinue the availability 
* of this software. By using this software, you agree to the additional terms and conditions found by accessing the 
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2018, 2022 Renesas Electronics Corporation. All rights reserved.
***********************************************************************************************************************/

/***********************************************************************************************************************
* File Name        : Config_ADCA0.c
* Component Version: 1.3.0
* Device(s)        : R7F701690
* Description      : This file implements device driver for Config_ADCA0.
***********************************************************************************************************************/
/***********************************************************************************************************************
Pragma directive
***********************************************************************************************************************/
/* Start user code for pragma. Do not edit comment generated here */
/* End user code. Do not edit comment generated here */

/***********************************************************************************************************************
Includes
***********************************************************************************************************************/
#include "r_cg_macrodriver.h"
#include "r_cg_userdefine.h"
#include "Config_ADCA0.h"
/* Start user code for include. Do not edit comment generated here */
/* End user code. Do not edit comment generated here */

/***********************************************************************************************************************
Global variables and functions
***********************************************************************************************************************/
extern volatile uint32_t g_cg_sync_read;
/* Start user code for global. Do not edit comment generated here */
/* End user code. Do not edit comment generated here */

/***********************************************************************************************************************
* Function Name: R_Config_ADCA0_Create
* Description  : This function initializes the ADC module.
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
void R_Config_ADCA0_Create(void)
{
    /* Disable ADC0 error interrupt (INTADCA0ERR) operation and clear request */
    INTC2.ICADCA0ERR.BIT.MKADCA0ERR = _INT_PROCESSING_DISABLED;
    INTC2.ICADCA0ERR.BIT.RFADCA0ERR = _INT_REQUEST_NOT_OCCUR;
    /* Disable ADC0 SG1 end interrupt (INTADCA0I0) operation and clear request */
    INTC1.ICADCA0I0.BIT.MKADCA0I0 = _INT_PROCESSING_DISABLED;
    INTC1.ICADCA0I0.BIT.RFADCA0I0 = _INT_REQUEST_NOT_OCCUR;
    /* Disable ADC0 SG2 end interrupt (INTADCA0I1) operation and clear request */
    INTC1.ICADCA0I1.BIT.MKADCA0I1 = _INT_PROCESSING_DISABLED;
    INTC1.ICADCA0I1.BIT.RFADCA0I1 = _INT_REQUEST_NOT_OCCUR;
    /* Disable ADC0 SG3 end interrupt (INTADCA0I2) operation and clear request */
    INTC1.ICADCA0I2.BIT.MKADCA0I2 = _INT_PROCESSING_DISABLED;
    INTC1.ICADCA0I2.BIT.RFADCA0I2 = _INT_REQUEST_NOT_OCCUR;
    /* Set ADC0 global setting */
    ADCA0.ADCR.UINT32 = _ADC_12_BIT_MODE;
    ADCA0.SMPCR.UINT32 = _ADC_24_CYCLES;
    ADCA0.SFTCR.UINT32 = _ADC_READ_CLEAR_DISABLED | _ADC_LIMIT_ERROR_INT_DISABLE | _ADC_OVERWRITE_ERROR_INT_DISABLE;
    ADCA0.ULLMTBR0.UINT32 = _ADC0_UPPER_LIMIT_TABLE0 | _ADC0_LOWER_LIMIT_TABLE0;
    ADCA0.ULLMTBR1.UINT32 = _ADC0_UPPER_LIMIT_TABLE1 | _ADC0_LOWER_LIMIT_TABLE1;
    ADCA0.ULLMTBR2.UINT32 = _ADC0_UPPER_LIMIT_TABLE2 | _ADC0_LOWER_LIMIT_TABLE2;
    ADCA0.ULLMTBR3.UINT32 = _ADC0_UPPER_LIMIT_TABLE3 | _ADC0_LOWER_LIMIT_TABLE3;
    ADCA0.ULLMTBR4.UINT32 = _ADC0_UPPER_LIMIT_TABLE4 | _ADC0_LOWER_LIMIT_TABLE4;
    ADCA0.ULLMTBR5.UINT32 = _ADC0_UPPER_LIMIT_TABLE5 | _ADC0_LOWER_LIMIT_TABLE5;
    ADCA0.ULLMTBR6.UINT32 = _ADC0_UPPER_LIMIT_TABLE6 | _ADC0_LOWER_LIMIT_TABLE6;
    ADCA0.ULLMTBR7.UINT32 = _ADC0_UPPER_LIMIT_TABLE7 | _ADC0_LOWER_LIMIT_TABLE7;
    /* Set ADC0 scan group setting */
    ADCA0.SGVCSP1.UINT32 = _ADC0_SG1_START_POINTER;
    ADCA0.SGVCEP1.UINT32 = _ADC0_SG1_END_POINTER;
    ADCA0.SGMCYCR1.UINT32 = _ADC_SG_MULTICYCLE_NUMBER_1;
    ADCA0.SGCR1.UINT32 = _ADC_SG_SCAN_MODE_MULTICYCLE | _ADC_SG_SCAN_END_INT_DISABLE | _ADC_SG_HW_TRIGGER_ENABLE;
    ADCA0.SGTSEL1.UINT32 = _ADC_SG_HW_TRIGGER_FACTOR_8;
    ADCA0.VCR00.UINT32 = _ADC_VIRTUAL_CHANNEL_END_INT_DISABLE | _ADC_LIMIT_TABLE_SELECT_NONE | 
                         _ADC_PHYSICAL_CHANNEL_AN00;
    ADCA0.VCR01.UINT32 = _ADC_VIRTUAL_CHANNEL_END_INT_DISABLE | _ADC_LIMIT_TABLE_SELECT_NONE | 
                         _ADC_PHYSICAL_CHANNEL_AN02;
    ADCA0.VCR02.UINT32 = _ADC_VIRTUAL_CHANNEL_END_INT_DISABLE | _ADC_LIMIT_TABLE_SELECT_NONE | 
                         _ADC_PHYSICAL_CHANNEL_AN04;
    ADCA0.VCR03.UINT32 = _ADC_VIRTUAL_CHANNEL_END_INT_DISABLE | _ADC_LIMIT_TABLE_SELECT_NONE | 
                         _ADC_PHYSICAL_CHANNEL_AN01;
    ADCA0.VCR04.UINT32 = _ADC_VIRTUAL_CHANNEL_END_INT_DISABLE | _ADC_LIMIT_TABLE_SELECT_NONE | 
                         _ADC_PHYSICAL_CHANNEL_AN05;
    ADCA0.VCR05.UINT32 = _ADC_VIRTUAL_CHANNEL_END_INT_DISABLE | _ADC_LIMIT_TABLE_SELECT_NONE | 
                         _ADC_PHYSICAL_CHANNEL_AN06;
    ADCA0.VCR06.UINT32 = _ADC_VIRTUAL_CHANNEL_END_INT_DISABLE | _ADC_LIMIT_TABLE_SELECT_NONE | 
                         _ADC_PHYSICAL_CHANNEL_AN07;
    ADCA0.VCR07.UINT32 = _ADC_VIRTUAL_CHANNEL_END_INT_DISABLE | _ADC_LIMIT_TABLE_SELECT_NONE | 
                         _ADC_PHYSICAL_CHANNEL_AN08;
    ADCA0.VCR08.UINT32 = _ADC_VIRTUAL_CHANNEL_END_INT_DISABLE | _ADC_LIMIT_TABLE_SELECT_NONE | 
                         _ADC_PHYSICAL_CHANNEL_AN09;
    ADCA0.VCR09.UINT32 = _ADC_VIRTUAL_CHANNEL_END_INT_DISABLE | _ADC_LIMIT_TABLE_SELECT_NONE | 
                         _ADC_PHYSICAL_CHANNEL_AN03;
    ADCA0.VCR10.UINT32 = _ADC_VIRTUAL_CHANNEL_END_INT_DISABLE | _ADC_LIMIT_TABLE_SELECT_NONE | 
                         _ADC_PHYSICAL_CHANNEL_AN24;
    /* Set T&H setting */
    ADCA0.THCR.UINT32 = _ADC_TH_SAMPLING_AUTOMATIC;
    ADCA0.THACR.UINT32 = _ADC_TH_SG_START_TRIGGER_USED | _ADC_TH_SELECT_SG1;
    ADCA0.THER.UINT32 = _ADC_TH4_ENABLED | _ADC_TH2_ENABLED | _ADC_TH0_ENABLED;
    ADCA0.THGSR.UINT32 = _ADC_TH2_GROUP_A | _ADC_TH0_GROUP_A;
    /* Synchronization processing */
    g_cg_sync_read = ADCA0.ADHALTR.UINT32;
    __syncp();
    /* Clear ADC0 error interrupt (INTADCA0ERR) request and enable operation */
    INTC2.ICADCA0ERR.BIT.RFADCA0ERR = _INT_REQUEST_NOT_OCCUR;
    INTC2.ICADCA0ERR.BIT.MKADCA0ERR = _INT_PROCESSING_ENABLED;

    R_Config_ADCA0_Create_UserInit();
}

/***********************************************************************************************************************
* Function Name: R_Config_ADCA0_Halt
* Description  : This function halts ADC0.
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
void R_Config_ADCA0_Halt(void)
{
    ADCA0.ADHALTR.UINT32 = _ADC_HALT;
    /* Synchronization processing */
    g_cg_sync_read = ADCA0.ADHALTR.UINT32;
    __syncp();
}

/***********************************************************************************************************************
* Function Name: R_Config_ADCA0_ScanGroup1_Start
* Description  : This function enables ADC0 scan group 1 end interrupt.
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
void R_Config_ADCA0_ScanGroup1_Start(void)
{
}

/***********************************************************************************************************************
* Function Name: R_Config_ADCA0_ScanGroup1_OperationOn
* Description  : This function starts ADC0 scan group 1 convertion.
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
void R_Config_ADCA0_ScanGroup1_OperationOn(void)
{
    /* Enable ADC0 SG1 operation */
    ADCA0.SGSTCR1.UINT32 = _ADC_START;
}

/***********************************************************************************************************************
* Function Name: R_Config_ADCA0_ScanGroup1_GetResult
* Description  : This function gets A/D conversion result for ADC0 scan group 1.
* Arguments    : buffer -
*                    buffer pointer
*                buffer_size -
*                    the size of buffer
* Return Value : MD_STATUS -
*                    MD_OK or MD_ARGERROR
***********************************************************************************************************************/
MD_STATUS R_Config_ADCA0_ScanGroup1_GetResult(uint16_t * const buffer, uint8_t buffer_size)
{
    uint8_t start_pointer;
    uint8_t end_pointer;
    uint8_t count_num;
    uint8_t buffer_index;
    uint32_t * p_addr;
    uint32_t convert_data;

    start_pointer = ADCA0.SGVCSP1.UINT32;
    end_pointer = ADCA0.SGVCEP1.UINT32;
    count_num = (end_pointer - start_pointer) + 1U;
    if (buffer_size < count_num)
    {
        return MD_ARGERROR;
    }

    p_addr = (uint32_t*)(_ADC0_VCHANNEL_DATA_BASE + (2U * start_pointer));

    for (buffer_index = 0U; buffer_index < count_num; buffer_index = buffer_index + 2U)
    {
        convert_data = (uint32_t) * p_addr;
        buffer[buffer_index] = (uint16_t)(convert_data & 0x0000FFFFUL);
        if ((count_num - buffer_index) > 1U)
        {
            buffer[buffer_index + 1U] = (uint16_t)((convert_data & 0xFFFF0000UL) >> 16U);
        }
        p_addr++;
    }

    return MD_OK;
}

/***********************************************************************************************************************
* Function Name: R_Config_ADCA0_TH_GroupA_Start
* Description  : This function starts ADC0 TH group A.
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
void R_Config_ADCA0_TH_GroupA_Start(void)
{
    /*Hold for TH group A is started*/
    ADCA0.THAHLDSTCR.UINT32 = _ADC_TH_SAMPLING_START;
}

/***********************************************************************************************************************
* Function Name: R_Config_ADCA0_TH_Sampling_Start
* Description  : This function starts of sampling to all TH.
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
void R_Config_ADCA0_TH_Sampling_Start(void)
{
    ADCA0.THSMPSTCR.UINT32 = _ADC_TH_SAMPLING_ALL_START;
}

/* Start user code for adding. Do not edit comment generated here */
/* End user code. Do not edit comment generated here */
